package com.sensor.iot.api.service.impl;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.sensor.iot.api.config.SensorIotApiProperties;
import com.sensor.iot.api.enums.ApiEnum;
import com.sensor.iot.api.interceptor.AuthInterceptor;
import com.sensor.iot.api.model.ApiResp;
import com.sensor.iot.api.model.PropQuery;
import com.sensor.iot.api.model.QueryResp;
import com.sensor.iot.api.model.TslHistoryQuery;
import com.sensor.iot.api.service.DeviceTslApiService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * DeviceTslApiServiceImpl
 *
 * @author Frank
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class DeviceTslApiServiceImpl implements DeviceTslApiService {
    private final SensorIotApiProperties apiProperties;
    private final AuthInterceptor authInterceptor;

    @Override
    public ApiResp<List<Map<String, Object>>> deviceProp(String deviceId) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.DEVICE_TSL_PROP.url(apiProperties.getBaseUrl(), deviceId))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<ApiResp<List<Map<String, Object>>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]获取设备物模型属性配置API错误:" + ex.getMessage());
        }
    }

    @Override
    public ApiResp<Map<String, Object>> deviceProp(String deviceId, String identity) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.DEVICE_TSL_PROP.url(apiProperties.getBaseUrl(), deviceId))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .form("identity", identity)
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<ApiResp<Map<String, Object>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]获取设备物模型属性配置API错误:" + ex.getMessage());
        }
    }

    @Override
    public ApiResp<List<Map<String, Object>>> deviceEvent(String deviceId) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.DEVICE_TSL_EVENT.url(apiProperties.getBaseUrl(), deviceId))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<ApiResp<List<Map<String, Object>>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]获取设备物模型事件配置API错误:" + ex.getMessage());
        }
    }

    @Override
    public ApiResp<Map<String, Object>> deviceEvent(String deviceId, String identity) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.DEVICE_TSL_EVENT.url(apiProperties.getBaseUrl(), deviceId))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .form("identity", identity)
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<ApiResp<Map<String, Object>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]获取设备物模型事件配置API错误:" + ex.getMessage());
        }
    }

    @Override
    public ApiResp<List<Map<String, Object>>> deviceFunc(String deviceId) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.DEVICE_TSL_FUNC.url(apiProperties.getBaseUrl(), deviceId))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<ApiResp<List<Map<String, Object>>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]获取设备物模型功能配置API错误:" + ex.getMessage());
        }
    }

    @Override
    public ApiResp<Map<String, Object>> deviceFunc(String deviceId, String identity) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.DEVICE_TSL_FUNC.url(apiProperties.getBaseUrl(), deviceId))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .form("identity", identity)
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<ApiResp<Map<String, Object>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]获取设备物模型功能配置API错误:" + ex.getMessage());
        }
    }

    @Override
    public ApiResp<Map<String, Object>> queryProp(PropQuery query) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.TSL_PROP.url(apiProperties.getBaseUrl()))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .form(query.toMap())
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<ApiResp<Map<String, Object>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]查询指定设备属性值API错误:" + ex.getMessage());
        }
    }

    @Override
    public ApiResp<Map<String, Object>> queryPropLatest(String deviceId) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.TSL_PROP_LATEST.url(apiProperties.getBaseUrl()))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .form("deviceId", deviceId)
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<ApiResp<Map<String, Object>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]查询指定设备最新属性数据API错误:" + ex.getMessage());
        }
    }

    @Override
    public QueryResp<Map<String, Object>> queryPropHistory(TslHistoryQuery query) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.TSL_PROP_HISTORY.url(apiProperties.getBaseUrl()))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .form(query.toMap())
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<QueryResp<Map<String, Object>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]查询设备属性历史数据API错误:" + ex.getMessage());
        }
    }

    @Override
    public QueryResp<Map<String, Object>> queryEventHistory(TslHistoryQuery query) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.TSL_EVENT_HISTORY.url(apiProperties.getBaseUrl()))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .form(query.toMap())
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<QueryResp<Map<String, Object>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]查询设备事件历史数据API错误:" + ex.getMessage());
        }
    }

    @Override
    public QueryResp<Map<String, Object>> queryFuncHistory(TslHistoryQuery query) {
        try (HttpResponse response = HttpRequest
                .get(ApiEnum.TSL_FUNC_HISTORY.url(apiProperties.getBaseUrl()))
                .addInterceptor(authInterceptor)
                .timeout(apiProperties.getTimeout())
                .form(query.toMap())
                .execute()
        ) {
            String result = response.body();
            return JSONObject.parseObject(result, new TypeReference<QueryResp<Map<String, Object>>>() {
            });
        } catch (Exception ex) {
            throw new RuntimeException("[IOT]查询设备功能操作历史API错误:" + ex.getMessage());
        }
    }
}
